/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

/*
 * Parts of this code are copyright:
 * 
 * The contents of this file are subject to the Joram Public License, as defined
 * by the file JORAM_LICENSE.TXT
 * 
 * You may not use this file except in compliance with the License. You may
 * obtain a copy of the License on the Objectweb web site (www.objectweb.org).
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 * 
 * The Original Code is Joram, including the java packages fr.dyade.aaa.agent,
 * fr.dyade.aaa.util, fr.dyade.aaa.ip, fr.dyade.aaa.mom, and fr.dyade.aaa.joram,
 * released May 24, 2000.
 * 
 * The Initial Developer of the Original Code is Dyade. The Original Code and
 * portions created by Dyade are Copyright Bull and Copyright INRIA. All Rights
 * Reserved.
 * 
 * The present code contributor is INRIA Rhone-Alpes.
 */

package org.objectweb.dream.aaa.agent;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * <code>Agent</code> used to allow remote agent creation. Every agent server
 * hosts a factory agent, they all use a predefined stamp identifier
 * <code>AgentId.factoryId</code>. The factory must be able to create all
 * types of objects, actually it is supposed that the corresponding classes can
 * be reached.
 * <p>
 * The agent creation process involves the following steps
 * <ul>
 * <li>locally creating the <code>Agent</code> object in memory ;
 * <li>serializing the agent state, building an <code>AgentCreateRequest</code>
 * notification with the resulting string, sending it to the target Factory
 * agent;
 * <li>the factory agent building the object in memory from the serialized
 * image, and saving it into operational storage;
 * <li>the factory agent calling the initialize method.
 * </ul>
 * The AgentDeleteRequest class of notification follows a similar process to
 * remotely delete agents.
 * 
 * @author ScalAgent
 */
public class AgentFactoryImpl extends AbstractComponent
    implements
      ComponentReact
{

  /**
   * the name of the client interface used to send reply notifications
   */
  public static final String PUSH_NOTIFICATION_ITF_NAME = "outPushNotification";

  /**
   * The name of the client interface binded to the repository component
   */
  public static final String REPOSITORY_ITF_NAME        = "repository";

  private Repository         repositoryItf;
  private PushNotification   pushNotificationItf;

  /**
   * Reacts to notifications ... .
   * 
   * @param from agent sending notification
   * @param not notification to react to
   * @exception Exception unspecialized exception
   */
  public void react(AgentId from, Notification not) throws Exception
  {
    AgentId id = AgentId.factoryId(repositoryItf.getRepositoryId());
    if (not instanceof AgentCreateRequest)
    {
      AgentCreateRequest cnot = (AgentCreateRequest) not;
      try
      {
        // Restore the new agent state.

        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(
            cnot.getAgentState(), 0, cnot.getAgentState().length));
        Agent ag = (Agent) ois.readObject();
        try
        {
          ois.close();
        }
        catch (IOException exc)
        {
          // ignored
        }

        createAgent(ag);

        if (logger.isLoggable(BasicLevel.DEBUG))
        {
          logger.log(BasicLevel.DEBUG, "create Agent" + ag.getId() + " ["
              + ag.getName() + "]");
        }
        if (cnot.getReply() != null)
        {
          AgentCreateReply reply = new AgentCreateReply(ag.getId());
          reply.setContext(cnot.getContext());
          pushNotificationItf.pushNotification(id, cnot.getReply(), reply);
        }
      }
      catch (Exception exc)
      {
        //  If there is an explicit reply request send it the
        // ExceptionNotification to the requester else to the
        // sender.
        cnot.resetAgentState();
        logger.log(BasicLevel.ERROR, "can't create Agent" + cnot.getDeploy(),
            exc);
        ExceptionNotification excNot = new ExceptionNotification(id, cnot, exc);
        excNot.setContext(cnot.getContext());
        if (cnot.getReply() != null)
        {
          pushNotificationItf.pushNotification(id, cnot.getReply(), excNot);
        }
        else
        {
          pushNotificationItf.pushNotification(id, from, excNot);
        }
      }
    }
    else if (not instanceof AgentDeleteRequest)
    {
      try
      {
        deleteAgent(from);
        if (((AgentDeleteRequest) not).getReply() != null)
        {
          pushNotificationItf.pushNotification(id, ((AgentDeleteRequest) not)
              .getReply(), new DeleteAck(from));
        }
      }
      catch (Exception exc)
      {
        logger.log(BasicLevel.DEBUG, "agent deletion error", exc);
        if (((AgentDeleteRequest) not).getReply() != null)
        {
          pushNotificationItf.pushNotification(id, ((AgentDeleteRequest) not)
              .getReply(), new DeleteAck(from, exc));
        }
      }
    }
    else
    {
      //      try {
      //        // super.react(from, not);
      //      } catch (Exception exc) {
      pushNotificationItf.pushNotification(id, from, new ExceptionNotification(
          id, not, new UnknownNotificationException()));
      //      }
    }
  }

  /**
   * Deletes an agent in the local agent server.
   */
  void deleteAgent(AgentId from) throws Exception
  {
    try
    {
      if (logger.isLoggable(BasicLevel.DEBUG))
      {
        logger.log(BasicLevel.DEBUG, "delete Agent" + from);
      }
      repositoryItf.removeAgent(from);
    }
    catch (UnknownAgentException exc)
    {
      logger.log(BasicLevel.ERROR, "can't delete unknown Agent" + from, exc);
      throw new Exception("Can't delete Agent" + from + " UnknowAgent");
    }
  }

  /**
   * Creates and initializes an agent in the local agent server.
   * <p>
   * This function is private to the package and is used only in the static
   * <code>init</code> function of the <code>Agent</code> class to create
   * system agents.
   * 
   * @param agent agent object to create
   * @exception Exception unspecialized exception
   */
  void createAgent(Agent agent) throws Exception
  {
    logger.log(BasicLevel.DEBUG, "createAgent()");
    repositoryItf.addAgent(agent.getId(), agent);
    agent.deployed = true;
    // Initialize the agent
    agent.initialize(true);
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{REPOSITORY_ITF_NAME, PUSH_NOTIFICATION_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (clientItfName.equals(REPOSITORY_ITF_NAME))
    {
      repositoryItf = (Repository) serverItf;
    }
    else if (clientItfName.equals(PUSH_NOTIFICATION_ITF_NAME))
    {
      pushNotificationItf = (PushNotification) serverItf;
    }
    super.bindFc(clientItfName, serverItf);
  }

}