/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.net.InetAddress;
import java.util.Iterator;

import org.objectweb.dream.aaa.agent.AgentId;
import org.objectweb.dream.aaa.agent.Notification;
import org.objectweb.dream.channel.IPChannelDestinationChunk;
import org.objectweb.dream.message.AbstractNonExtensibleMessage;
import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageType;
import org.objectweb.dream.message.chunk.ArrowChunk;
import org.objectweb.dream.protocol.causality.CausalityChunk;
import org.objectweb.dream.protocol.causality.Update;
import org.objectweb.dream.util.ArrayIterator;

/**
 * The specific and unextensible message class used in AAA. This message
 * contains 4 chunks :{@link AgentChunk},{@link ArrowChunk},
 * {@link CausalityChunk }and {@link IPChannelDestinationChunk}. This class
 * also implements {@link MessageType}.
 */
public class AAAMessage extends AbstractNonExtensibleMessage
    implements
      Message,
      MessageType,
      AgentChunk,
      ArrowChunk,
      CausalityChunk,
      IPChannelDestinationChunk,
      Externalizable
{

  /** the type of messages of this class. */
  public static final MessageType TYPE                              = new AAAMessage();

  private static final String     AGENT_CHUNK_NAME                  = AgentChunk.DEFAULT_NAME;

  private static final String     ARROW_CHUNK_NAME                  = "CausalProcesses";

  private static final String     CAUSALITY_CHUNK_NAME              = "Causality";

  private static final String     IP_CHANNEL_DESTINATION_CHUNK_NAME = "IPChannelDestination";

  private static final String[]   CHUNK_NAMES                       = {
      AGENT_CHUNK_NAME, ARROW_CHUNK_NAME, CAUSALITY_CHUNK_NAME,
      IP_CHANNEL_DESTINATION_CHUNK_NAME                             };

  // fields of AgentChunk
  private transient AgentId       from;

  private transient AgentId       to;

  private transient Notification  notif;

  // fields of ArrowChunk
  private transient short         processFrom;

  private transient short         processTo;

  // fields of CausalityChunk
  private transient Object        stamp;

  // fields of IPChannelDestinationChunk
  private transient InetAddress   addr;

  private transient int           port;

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of the Message interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see Message#transfertChunkStates(Message)
   */
  public void transfertChunkStates(Message newInstance)
  {
    AgentChunk ac = (AgentChunk) newInstance.getChunk(AGENT_CHUNK_NAME);
    ac.setAgentFrom(getAgentFrom());
    ac.setAgentTo(getAgentFrom());
    if (getAgentNotification() != null)
    {
      ac.setAgentNotifiaction((Notification) getAgentNotification().clone());
    }
    else
    {
      ac.setAgentNotifiaction(null);
    }

    ArrowChunk arc = (ArrowChunk) newInstance.getChunk(ARROW_CHUNK_NAME);
    arc.setArrowFrom(getArrowFrom());
    arc.setArrowTo(getArrowTo());

    CausalityChunk cr = (CausalityChunk) newInstance
        .getChunk(CAUSALITY_CHUNK_NAME);
    cr.setCausalityStamp(getCausalityStamp());

    IPChannelDestinationChunk ipcc = (IPChannelDestinationChunk) newInstance
        .getChunk(IP_CHANNEL_DESTINATION_CHUNK_NAME);
    ipcc.setChannelDestinationAddr(getChannelDestinationAddr());
    ipcc.setChannelDestinationPort(getChannelDestinationPort());
  }

  /**
   * @see Message#recycle()
   */
  public void recycle()
  {
    from = null;
    to = null;
    notif = null;
    processFrom = 0;
    processTo = 0;
    stamp = null;
    addr = null;
    port = 0;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of the MessageType interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see MessageType#getChunkType(String)
   */
  public ChunkType getChunkType(String name)
  {
    if (name.equals(AGENT_CHUNK_NAME))
    {
      return AgentChunk.TYPE;
    }
    else if (name.equals(ARROW_CHUNK_NAME))
    {
      return ArrowChunk.TYPE;
    }
    else if (name.equals(CAUSALITY_CHUNK_NAME))
    {
      return CausalityChunk.TYPE;
    }
    else if (name.equals(IP_CHANNEL_DESTINATION_CHUNK_NAME))
    {
      return IPChannelDestinationChunk.TYPE;
    }
    return null;
  }

  /**
   * @see MessageType#getChunkNames()
   */
  public String[] getChunkNames()
  {
    return CHUNK_NAMES;
  }

  /**
   * @see MessageType#getChunkNamesIterator()
   */
  public Iterator getChunkNamesIterator()
  {
    return new ArrayIterator(CHUNK_NAMES);
  }

  /**
   * @see MessageType#isEmpty()
   */
  public boolean isEmpty()
  {
    return false;
  }

  /**
   * @see MessageType#isSubTypeOf(MessageType)
   */
  public boolean isSubTypeOf(MessageType t)
  {
    // NOT IMPLEMENTED message subtyping is not yet implemented
    return false;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  implementation of AgentChunk interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see AgentChunk#getAgentFrom()
   */
  public AgentId getAgentFrom()
  {
    return from;
  }

  /**
   * @see AgentChunk#setAgentFrom(AgentId)
   */
  public void setAgentFrom(AgentId id)
  {
    from = id;
  }

  /**
   * @see AgentChunk#getAgentTo()
   */
  public AgentId getAgentTo()
  {
    return to;
  }

  /**
   * @see AgentChunk#setAgentTo(AgentId)
   */
  public void setAgentTo(AgentId id)
  {
    to = id;
  }

  /**
   * @see AgentChunk#getAgentNotification()
   */
  public Notification getAgentNotification()
  {
    return notif;
  }

  /**
   * @see AgentChunk#setAgentNotifiaction(Notification)
   */
  public void setAgentNotifiaction(Notification not)
  {
    notif = not;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of CausalityChunk Interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see CausalityChunk#getCausalityStamp()
   */
  public Object getCausalityStamp()
  {
    return stamp;
  }

  /**
   * @see CausalityChunk#setCausalityStamp(java.lang.Object)
   */
  public void setCausalityStamp(Object stamp)
  {
    this.stamp = stamp;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of IPChannelChunk interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see IPChannelDestinationChunk#getChannelDestinationAddr()
   */
  public InetAddress getChannelDestinationAddr()
  {
    return addr;
  }

  /**
   * @see IPChannelDestinationChunk#setChannelDestinationAddr(InetAddress)
   */
  public void setChannelDestinationAddr(InetAddress addr)
  {
    this.addr = addr;
  }

  /**
   * @see IPChannelDestinationChunk#getChannelDestinationPort()
   */
  public int getChannelDestinationPort()
  {
    return port;
  }

  /**
   * @see IPChannelDestinationChunk#setChannelDestinationPort(int)
   */
  public void setChannelDestinationPort(int port)
  {
    this.port = port;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  implementation of ArrowChunk interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see ArrowChunk#getArrowFrom()
   */
  public short getArrowFrom()
  {
    return processFrom;
  }

  /**
   * @see ArrowChunk#setArrowFrom(short)
   */
  public void setArrowFrom(short from)
  {
    processFrom = from;
  }

  /**
   * @see ArrowChunk#getArrowTo()
   */
  public short getArrowTo()
  {
    return processTo;
  }

  /**
   * @see ArrowChunk#setArrowTo(short)
   */
  public void setArrowTo(short to)
  {
    this.processTo = to;
  }

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of Externalizable interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    // read from
    from = new AgentId(in.readShort(), in.readShort(), in.readInt());

    // read to
    to = new AgentId(in.readShort(), in.readShort(), in.readInt());

    // read notification
    notif = (Notification) in.readObject();

    // read processFrom
    processFrom = in.readShort();

    // read processTo
    processTo = in.readShort();

    // read stamp
    if (in.readBoolean())
    {
      stamp = Update.readUpdate(in);
    }
    else
    {
      stamp = in.readObject();
    }
  }

  /**
   * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException
  {
    // write from
    out.writeShort(from.getFrom());
    out.writeShort(from.getTo());
    out.writeInt(from.getStamp());

    // write to
    out.writeShort(to.getFrom());
    out.writeShort(to.getTo());
    out.writeInt(to.getStamp());

    // write notif
    out.writeObject(notif);

    // write processFrom
    out.writeShort(processFrom);

    // write processTo
    out.writeShort(processTo);

    // write stamp
    if ((stamp != null) && (stamp instanceof Update))
    {
      out.writeBoolean(true);
      ((Update) stamp).writeExternal(out);
    }
    else
    {
      out.writeBoolean(false);
      out.writeObject(stamp);
    }
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer(super.toString());
    sb.append(" from=").append(getAgentFrom());
    sb.append(" to=").append(getAgentTo());
    sb.append(" not=").append(getAgentNotification());
    sb.append(" from=").append(getArrowFrom());
    sb.append(" to=").append(getArrowTo());
    if (getChannelDestinationAddr() != null)
    {
      sb.append(" destination=").append(getChannelDestinationAddr());
      sb.append(":").append(getChannelDestinationPort());
    }

    return sb.toString();
  }
}