/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components;

import java.util.Map;

import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.PushPushDreamComponent;
import org.objectweb.dream.aaa.components.conf.AgentServerConfiguration;
import org.objectweb.dream.aaa.components.conf.RepositoryDesc;
import org.objectweb.dream.aaa.components.conf.ServerDesc;
import org.objectweb.dream.channel.IPChannelDestinationChunk;
import org.objectweb.dream.message.ChunkAlreadyExistException;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This transformer adds a {@link IPChannelDestinationChunk }chunk to the
 * message. The name of the added chunk is <code>"IPChannelDestination"</code>.
 */
public class BasicDestinationResolverImpl extends PushPushDreamComponent
{

  private MessageManager           messageManagerItf;

  private AgentServerConfiguration agentServerConfigurationItf;

  /**
   * @see Push#push(Message, Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    AgentChunk agentChunk = (AgentChunk) message
        .getChunk(AgentChunk.DEFAULT_NAME);
    if (agentChunk == null)
    {
      logger.log(BasicLevel.ERROR, "Unable to find agent chunk named "
          + AgentChunk.DEFAULT_NAME + ". The message is dropped");
      return;
    }

    IPChannelDestinationChunk destChunk = (IPChannelDestinationChunk) message
        .getChunk("IPChannelDestination");
    if (destChunk == null)
    {
      if (message instanceof ExtensibleMessage)
      {
        destChunk = (IPChannelDestinationChunk) messageManagerItf
            .createChunk(IPChannelDestinationChunk.TYPE);
        try
        {
          ((ExtensibleMessage) message).addChunk("IPChannelDestination",
              IPChannelDestinationChunk.TYPE, destChunk);
        }
        catch (ChunkAlreadyExistException e1)
        {
          Error.bug(logger, e1);
        }
      }
      else
      {
        logger
            .log(
                BasicLevel.ERROR,
                "Unable to find ipChannelDestination chunk in the unextensible message. The message is dropped");
        return;
      }
    }

    short regId = agentChunk.getAgentTo().getTo();
    RepositoryDesc reg = agentServerConfigurationItf.getRepositoryDesc(regId);
    if (reg == null)
    {
      logger.log(BasicLevel.ERROR,
          "Unknown repository identifier, message dropped");
      messageManagerItf.deleteMessage(message);
      return;
    }
    ServerDesc server = reg.getServer();
    destChunk.setChannelDestinationAddr(server.getAccessNetwork().getServer()
        .getInetAddress());
    destChunk.setChannelDestinationPort(server.getAccessNetwork().getPort());

    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      logger.log(BasicLevel.DEBUG, "Push message " + message);
    }
    try
    {
      outPushItf.push(message, context);
    }
    catch (PushException e)
    {
      messageManagerItf.deleteMessage(message);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME,
        AgentServerConfiguration.DEFAULT_NAME};
  }

  /**
   * Stores the <code>outPush</code> binding.
   * 
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    else if (clientItfName.equals(AgentServerConfiguration.DEFAULT_NAME))
    {
      agentServerConfigurationItf = (AgentServerConfiguration) serverItf;
    }
  }
}