/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aaa.components.conf;

/**
 * Network description.
 */
public final class NetworkDesc implements Comparable
{

  /** The domain to which this network belong. */
  private DomainDesc domain;

  /** The server to which this network belong. */
  private ServerDesc server;

  /** The name of the ADL of the network. */
  private String     adl;

  /** The index of this network in its domain. */
  private short      index = -1;

  /** The port number on which this NetworkIn will listen. */
  private int        port;

  /**
   * Constructor
   * 
   * @param server the server to which this network belong.
   * @param domain the domain to which this network belong.
   * @param adl the name of the ADL of the network.
   * @param port the port number on which this NetworkIn will listen.
   */
  public NetworkDesc(ServerDesc server, DomainDesc domain, String adl, int port)
  {
    this.domain = domain;
    this.server = server;
    this.adl = adl;
    this.port = port;
  }

  /**
   * returns the port number on which this NetworkIn will listen.
   * 
   * @return the port number on which this NetworkIn will listen.
   */
  public int getPort()
  {
    return port;
  }

  /**
   * Returns the name of the ADL of the network.
   * 
   * @return the name of the ADL of the network.
   */
  public String getAdl()
  {
    return adl;
  }

  /**
   * Returns the domain to which this network belong.
   * 
   * @return the domain to which this network belong.
   */
  public DomainDesc getDomain()
  {
    return domain;
  }

  /**
   * Returns the server to which this network belong.
   * 
   * @return the server to which this network belong.
   */
  public ServerDesc getServer()
  {
    return server;
  }

  /**
   * Returns the index of this server in the causality matrix.
   * 
   * @return the index of this server in the causality matrix.
   */
  public short getDomainIndex()
  {
    return index;
  }

  /**
   * Set the index of this network in its domain.
   * 
   * @param index the index of this network in its domain.
   */
  public void setDomainIndex(short index)
  {
    this.index = index;
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    return "{domain=" + domain.getName() + ", port=" + getPort() + "}";
  }

  /**
   * Compare server names lexicographically.
   * 
   * @see java.lang.Comparable#compareTo(java.lang.Object)
   */
  public int compareTo(Object o)
  {
    return getServer().getName().compareTo(
        ((NetworkDesc) o).getServer().getName());
  }
}