/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.aggregator;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.BasicMessageType;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic implementation of a Push/Pull aggregator. Each pushed message is stored
 * in an aggregated message. This aggregated message is returned by the pull
 * method, which then creates next aggregated message to be returned.
 */
public class BasicPushPullAggregatorImpl extends AbstractComponent
    implements
      Push,
      Pull
{

  /** the message manager client interface of this component. */
  protected MessageManager    messageManagerItf;

  /**
   * The aggregated message that will be returned by the next call of the pull
   * method.
   */
  protected ExtensibleMessage nextAggregatedMessage = null;

  /**
   * @see Push#push(Message, Map)
   */
  public synchronized void push(Message message, Map context)
      throws PushException
  {
    if (nextAggregatedMessage == null)
    {
      nextAggregatedMessage = (ExtensibleMessage) messageManagerItf
          .createMessage(BasicMessageType.EMPTY_MESSAGE_TYPE);
    }
    nextAggregatedMessage.addSubMessage(message);
  }

  /**
   * @see Pull#pull(Map)
   */
  public synchronized Message pull(Map context) throws PullException
  {
    if (nextAggregatedMessage == null)
    {
      return (ExtensibleMessage) messageManagerItf
          .createMessage(BasicMessageType.EMPTY_MESSAGE_TYPE);
    }
    else
    {
      Message msg = nextAggregatedMessage;
      nextAggregatedMessage = null;
      return msg;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

}