/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic implementation of a message codec. It encodes or decodes messages using
 * {@link ObjectOutput }and {@link ObjectInput }.
 */
public class BasicStreamMessageCodecImpl extends AbstractComponent
    implements
      MessageCodec
{

  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the MessageCodec interface
  // ---------------------------------------------------------------------------

  /**
   * Can decode message from an {@link ObjectInput }or from an
   * {@link InputStream}. If the given stream is an {@link InputStream}, an
   * {@link ObjectInputStream }is created, the message is red, then the
   * {@link ObjectInputStream }is closed.
   * 
   * @see MessageCodec#decode(Object)
   */
  public Message decode(Object stream) throws IOException
  {
    Object o;
    if (stream instanceof ObjectInput)
    {
      try
      {
        o = ((ObjectInput) stream).readObject();
      }
      catch (ClassNotFoundException e)
      {
        throw new IOException("Invalid ObjectInput stream : "
            + e.getLocalizedMessage());
      }
    }
    else if (stream instanceof InputStream)
    {
      ObjectInputStream ois = new ObjectInputStream((InputStream) stream);
      try
      {
        o = ois.readObject();
      }
      catch (ClassNotFoundException e)
      {
        throw new IOException("Invalid ObjectInput stream : "
            + e.getLocalizedMessage());
      }
      ois.close();
    }
    else
    {
      throw new IOException("Unknown stream");
    }
    if (!(o instanceof Message))
    {
      throw new IOException("Invalide incomming object : not a Message");
    }
    Message m = (Message) o;
    m.setMessageManagerId(messageManagerItf.getMessageManagerId());
    return m;
  }

  /**
   * Can encode message to an {@link ObjectOutput }or to an
   * {@link OutputStream}. If the given stream is an {@link OutputStream}, an
   * {@link ObjectOutputStream }is created, the message is written, then the
   * {@link ObjectOutputStream }is closed.
   * 
   * @see MessageCodec#encode(Object, Message)
   */
  public void encode(Object stream, Message message) throws IOException
  {
    if (stream instanceof ObjectOutput)
    {
      ((ObjectOutput) stream).writeObject(message);
    }
    else if (stream instanceof OutputStream)
    {
      ObjectOutputStream oos = new ObjectOutputStream((OutputStream) stream);
      oos.writeObject(message);
    }
    else
    {
      throw new IOException("Unknown stream");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    super.bindFc(clientItfName, serverItf);
  }

}